@@ -183,7 +183,7 @@ extern "C" {
 #define WTAP_ENCAP_SITA                         100
 #define WTAP_ENCAP_SCCP                         101
 #define WTAP_ENCAP_BLUETOOTH_HCI                102 /*raw packets without a transport layer header e.g. H4*/
-#define WTAP_ENCAP_IPMB                         103
+#define WTAP_ENCAP_IPMB_KONTRON                 103
 #define WTAP_ENCAP_IEEE802_15_4                 104
 #define WTAP_ENCAP_X2E_XORAYA                   105
 #define WTAP_ENCAP_FLEXRAY                      106
@@ -192,7 +192,7 @@ extern "C" {
 #define WTAP_ENCAP_CAN20B                       109
 #define WTAP_ENCAP_LAYER1_EVENT                 110
 #define WTAP_ENCAP_X2E_SERIAL                   111
-#define WTAP_ENCAP_I2C                          112
+#define WTAP_ENCAP_I2C_LINUX                    112
 #define WTAP_ENCAP_IEEE802_15_4_NONASK_PHY      113
 #define WTAP_ENCAP_TNEF                         114
 #define WTAP_ENCAP_USB_LINUX_MMAPPED            115
@@ -287,6 +287,11 @@ extern "C" {
 #define WTAP_ENCAP_EBHSCR                       204
 #define WTAP_ENCAP_VPP                          205
 #define WTAP_ENCAP_IEEE802_15_4_TAP             206
+#define WTAP_ENCAP_LOG_3GPP                     207
+#define WTAP_ENCAP_USB_2_0                      208
+#define WTAP_ENCAP_MP4                          209
+#define WTAP_ENCAP_SLL2                         210
+#define WTAP_ENCAP_ZWAVE_SERIAL                 211
 
 /* After adding new item here, please also add new item to encap_table_base array */
 
@@ -382,6 +387,8 @@ extern "C" {
 #define WTAP_FILE_TYPE_SUBTYPE_RFC7468                       82
 #define WTAP_FILE_TYPE_SUBTYPE_RUBY_MARSHAL                  83
 #define WTAP_FILE_TYPE_SUBTYPE_SYSTEMD_JOURNAL               84
+#define WTAP_FILE_TYPE_SUBTYPE_LOG_3GPP                      85
+#define WTAP_FILE_TYPE_SUBTYPE_MP4                           86
 
 #define WTAP_NUM_FILE_TYPES_SUBTYPES  wtap_get_num_file_types_subtypes()
 
@@ -412,7 +419,7 @@ extern "C" {
  *
  * For WTAP_ENCAP_USBPCAP, the maximum is 128MiB, as per
  *
- *    https://bugs.wireshark.org/bugzilla/show_bug.cgi?id=15985
+ *    https://gitlab.com/wireshark/wireshark/-/issues/15985
  *
  * We don't want to write out files that specify a maximum packet size
  * greater than 262144 if we don't have to, as software reading those
@@ -675,7 +682,7 @@ struct ieee_802_11a {
  *    http://wifi-insider.com/atheros/turbo.htm
  */
 #define PHDR_802_11A_TURBO_TYPE_NORMAL           0
-#define PHDR_802_11A_TURBO_TYPE_TURBO            1  /* If we don't know wehther it's static or dynamic */
+#define PHDR_802_11A_TURBO_TYPE_TURBO            1  /* If we don't know whether it's static or dynamic */
 #define PHDR_802_11A_TURBO_TYPE_DYNAMIC_TURBO    2
 #define PHDR_802_11A_TURBO_TYPE_STATIC_TURBO     3
 
@@ -800,6 +807,21 @@ struct ieee_802_11ad {
     guint8   mcs;            /* MCS index */
 };
 
+/*
+ * 802.11ax (HE).
+ */
+struct ieee_802_11ax {
+    /* Which of this information is present? */
+    guint    has_mcs_index:1;
+    guint    has_bwru:1;
+    guint    has_gi:1;
+
+    guint8   nsts:4;         /* Number of Space-time Streams */
+    guint8   mcs:4;          /* MCS index */
+    guint8   bwru:4;         /* Bandwidth/RU allocation */
+    guint8   gi:2;           /* Guard Interval */
+};
+
 union ieee_802_11_phy_info {
     struct ieee_802_11_fhss info_11_fhss;
     struct ieee_802_11b info_11b;
@@ -808,12 +830,14 @@ union ieee_802_11_phy_info {
     struct ieee_802_11n info_11n;
     struct ieee_802_11ac info_11ac;
     struct ieee_802_11ad info_11ad;
+    struct ieee_802_11ax info_11ax;
 };
 
 struct ieee_802_11_phdr {
     gint     fcs_len;          /* Number of bytes of FCS - -1 means "unknown" */
-    gboolean decrypted;        /* TRUE if frame is decrypted even if "protected" bit is set */
-    gboolean datapad;          /* TRUE if frame has padding between 802.11 header and payload */
+    guint    decrypted:1;      /* TRUE if frame is decrypted even if "protected" bit is set */
+    guint    datapad:1;        /* TRUE if frame has padding between 802.11 header and payload */
+    guint    no_a_msdus:1;     /* TRUE if we should ignore the A-MSDU bit */
     guint    phy;              /* PHY type */
     union ieee_802_11_phy_info phy_info;
 
@@ -919,7 +943,7 @@ struct nettl_phdr {
     guint32 devid;
     guint32 kind;
     gint32  pid;
-    guint16 uid;
+    guint32 uid;
 };
 
 /* Packet "pseudo-header" for MTP2 files. */
@@ -1091,6 +1115,7 @@ struct bthci_phdr {
 #define BTHCI_CHANNEL_ACL      2
 #define BTHCI_CHANNEL_SCO      3
 #define BTHCI_CHANNEL_EVENT    4
+#define BTHCI_CHANNEL_ISO      5
 
 /* pseudo header for WTAP_ENCAP_BLUETOOTH_LINUX_MONITOR */
 struct btmon_phdr {
@@ -1239,7 +1264,7 @@ union wtap_pseudo_header {
  * They may have a time stamp, and should be dissected and displayed
  * just as packets are.
  *
- * We distingiush between "events" and "reports" so that, for example,
+ * We distinguish between "events" and "reports" so that, for example,
  * the packet display can show the delta between a packet and an event
  * but not show the delta between a packet and a report, as the time
  * stamp of a report may not correspond to anything interesting on
@@ -1271,6 +1296,7 @@ union wtap_pseudo_header {
 #define REC_TYPE_FT_SPECIFIC_EVENT    1    /**< file-type-specific event */
 #define REC_TYPE_FT_SPECIFIC_REPORT   2    /**< file-type-specific report */
 #define REC_TYPE_SYSCALL              3    /**< system call */
+#define REC_TYPE_SYSTEMD_JOURNAL      4    /**< systemd journal entry */
 
 typedef struct {
     guint32   caplen;           /* data length in the file */
@@ -1282,6 +1308,8 @@ typedef struct {
     guint64   drop_count;       /* number of packets lost (by the interface and the
                                    operating system) between this packet and the preceding one. */
     guint32   pack_flags;       /* various flags, as per pcapng EPB */
+    guint32   interface_queue;  /* queue of the interface the packet was received on. */
+    guint64   packet_id;        /* unique packet identifier */
 
     union wtap_pseudo_header  pseudo_header;
 } wtap_packet_header;
@@ -1375,6 +1403,10 @@ typedef struct {
     /* ... Event ... */
 } wtap_syscall_header;
 
+typedef struct {
+    guint32   record_len;       /* length of the record */
+} wtap_systemd_journal_header;
+
 typedef struct {
     guint     rec_type;         /* what type of record is this? */
     guint32   presence_flags;   /* what stuff do we have? */
@@ -1384,13 +1416,18 @@ typedef struct {
         wtap_packet_header packet_header;
         wtap_ft_specific_header ft_specific_header;
         wtap_syscall_header syscall_header;
+        wtap_systemd_journal_header systemd_journal_header;
     } rec_header;
     /*
      * XXX - this should become a full set of options.
      */
     gchar     *opt_comment;     /* NULL if not available */
     gboolean  has_comment_changed; /* TRUE if the comment has been changed. Currently only valid while dumping. */
-
+    GPtrArray *packet_verdict;     /* packet verdicts. It would have made more
+                                      sense to put this in packet_header above
+                                      but due to the way the current code is
+                                      reusing the wtap_rec structure, it's
+                                      impossible to nicely clean it up. */
     /*
      * We use a Buffer so that we don't have to allocate and free
      * a buffer for the options for each record.
@@ -1426,6 +1463,9 @@ typedef struct {
 #define WTAP_HAS_COMMENTS      0x00000008  /**< comments */
 #define WTAP_HAS_DROP_COUNT    0x00000010  /**< drop count */
 #define WTAP_HAS_PACK_FLAGS    0x00000020  /**< packet flags */
+#define WTAP_HAS_PACKET_ID     0x00000040  /**< packet id */
+#define WTAP_HAS_INT_QUEUE     0x00000080  /**< interface queue */
+#define WTAP_HAS_VERDICT       0x00000100  /**< packet verdict */
 
 /**
  * Holds the required data from pcapng:s Section Header block(SHB).
@@ -1510,7 +1550,7 @@ typedef struct hashipv6 {
 } hashipv6_t;
 
 /** A struct with lists of resolved addresses.
- *  Used when writing name resoultion blocks (NRB)
+ *  Used when writing name resolutions blocks (NRB)
  */
 typedef struct addrinfo_lists {
     GList      *ipv4_addr_list; /**< A list of resolved hashipv4_t*/
@@ -1534,6 +1574,7 @@ typedef struct addrinfo_lists {
 typedef struct wtap_dump_params {
     int         encap;                      /**< Per-file packet encapsulation, or WTAP_ENCAP_PER_PACKET */
     int         snaplen;                    /**< Per-file snapshot length (what if it's per-interface?) */
+    int         tsprec;                     /**< Per-file time stamp precision */
     GArray     *shb_hdrs;                   /**< The section header block(s) information, or NULL. */
     wtapng_iface_descriptions_t *idb_inf;   /**< The interface description information, or NULL. */
     GArray     *nrb_hdrs;                   /**< The name resolution blocks(s) comment/custom_opts information, or NULL. */
@@ -1541,6 +1582,7 @@ typedef struct wtap_dump_params {
     const GArray *dsbs_growing;             /**< DSBs that will be written while writing packets, or NULL.
                                                  This array may grow since the dumper was opened and will subsequently
                                                  be written before newer packets are written in wtap_dump. */
+    gboolean    dont_copy_idbs;             /**< XXX - don't copy IDBs; this should eventually always be the case. */
 } wtap_dump_params;
 
 /* Zero-initializer for wtap_dump_params. */
@@ -1618,7 +1660,7 @@ struct file_extension_info {
  *
  * Note that the routine does not have to free the private data pointer on
  * error. The caller takes care of that by calling wtap_close on error.
- * (See https://bugs.wireshark.org/bugzilla/show_bug.cgi?id=8518)
+ * (See https://gitlab.com/wireshark/wireshark/-/issues/8518)
  *
  * However, the caller does have to free the private data pointer when
  * returning WTAP_OPEN_NOT_MINE, since the next file type will be called
@@ -1713,7 +1755,7 @@ struct file_type_subtype_info {
 
     /* the function to open the capture file for writing */
     /* should be NULL is this file type don't have write support */
-    int (*dump_open)(wtap_dumper *, int *);
+    int (*dump_open)(wtap_dumper *, int *, gchar **);
 
     /* if can_write_encap returned WTAP_ERR_CHECK_WSLUA, then this is used instead */
     /* this should be NULL for everyone except Lua-based file writers */
@@ -1735,9 +1777,9 @@ void wtap_init(gboolean load_wiretap_plugins);
  *
  * @param filename Name of the file to open
  * @param type WTAP_TYPE_AUTO for automatic recognize file format or explicit choose format type
- * @param err a positive "errno" value if the capture file can't be opened;
+ * @param[out] err a positive "errno" value if the capture file can't be opened;
  * a negative number, indicating the type of error, on other failures.
- * @param err_info for some errors, a string giving more details of
+ * @param[out] err_info for some errors, a string giving more details of
  * the error
  * @param do_random TRUE if random access to the file will be done,
  * FALSE if not
@@ -1774,24 +1816,45 @@ typedef void (*wtap_new_secrets_callback_t)(guint32 secrets_type, const void *se
 WS_DLL_PUBLIC
 void wtap_set_cb_new_secrets(wtap *wth, wtap_new_secrets_callback_t add_new_secrets);
 
-/** Returns TRUE if read was successful. FALSE if failure. data_offset is
- * set to the offset in the file where the data for the read packet is
- * located. */
+/** Read the next record in the file, filling in *phdr and *buf.
+ *
+ * @wth a wtap * returned by a call that opened a file for reading.
+ * @rec a pointer to a wtap_rec, filled in with information about the
+ * record.
+ * @buf a pointer to a Buffer, filled in with data from the record.
+ * @param err a positive "errno" value, or a negative number indicating
+ * the type of error, if the read failed.
+ * @param err_info for some errors, a string giving more details of
+ * the error
+ * @param offset a pointer to a gint64, set to the offset in the file
+ * that should be used on calls to wtap_seek_read() to reread that record,
+ * if the read succeeded.
+ * @return TRUE on success, FALSE on failure.
+ */
 WS_DLL_PUBLIC
-gboolean wtap_read(wtap *wth, int *err, gchar **err_info,
-    gint64 *data_offset);
+gboolean wtap_read(wtap *wth, wtap_rec *rec, Buffer *buf, int *err,
+    gchar **err_info, gint64 *offset);
 
+/** Read the record at a specified offset in a capture file, filling in
+ * *phdr and *buf.
+ *
+ * @wth a wtap * returned by a call that opened a file for random-access
+ * reading.
+ * @seek_off a gint64 giving an offset value returned by a previous
+ * wtap_read() call.
+ * @phdr a pointer to a struct wtap_pkthdr, filled in with information
+ * about the record.
+ * @buf a pointer to a Buffer, filled in with data from the record.
+ * @param err a positive "errno" value, or a negative number indicating
+ * the type of error, if the read failed.
+ * @param err_info for some errors, a string giving more details of
+ * the error
+ * @return TRUE on success, FALSE on failure.
+ */
 WS_DLL_PUBLIC
 gboolean wtap_seek_read(wtap *wth, gint64 seek_off, wtap_rec *rec,
     Buffer *buf, int *err, gchar **err_info);
 
-/*** get various information snippets about the current record ***/
-WS_DLL_PUBLIC
-wtap_rec *wtap_get_rec(wtap *wth);
-
-WS_DLL_PUBLIC
-guint8 *wtap_get_buf_ptr(wtap *wth);
-
 /*** initialize a wtap_rec structure ***/
 WS_DLL_PUBLIC
 void wtap_rec_init(wtap_rec *rec);
@@ -1834,20 +1897,30 @@ int wtap_file_encap(wtap *wth);
 WS_DLL_PUBLIC
 int wtap_file_tsprec(wtap *wth);
 
+/**
+ * @brief Gets number of section header blocks.
+ * @details Returns the number of existing SHBs.
+ *
+ * @param wth The wiretap session.
+ * @return The number of existing section headers.
+ */
+WS_DLL_PUBLIC
+guint wtap_file_get_num_shbs(wtap *wth);
+
 /**
  * @brief Gets existing section header block, not for new file.
- * @details Returns the pointer to the existing SHB, without creating a
+ * @details Returns the pointer to an existing SHB, without creating a
  *          new one. This should only be used for accessing info, not
  *          for creating a new file based on existing SHB info. Use
  *          wtap_file_get_shb_for_new_file() for that.
  *
  * @param wth The wiretap session.
- * @return The existing section header, which must NOT be g_free'd.
- *
- * XXX - need to be updated to handle multiple SHBs.
+ * @param shb_num The ordinal number (0-based) of the section header
+ * in the file
+ * @return The specified existing section header, which must NOT be g_free'd.
  */
 WS_DLL_PUBLIC
-wtap_block_t wtap_file_get_shb(wtap *wth);
+wtap_block_t wtap_file_get_shb(wtap *wth, guint shb_num);
 
 /**
  * @brief Gets new section header block for new file, based on existing info.
@@ -1878,6 +1951,20 @@ GArray* wtap_file_get_shb_for_new_file(wtap *wth);
 WS_DLL_PUBLIC
 void wtap_write_shb_comment(wtap *wth, gchar *comment);
 
+/**
+ * @brief Generate an IDB, given a wiretap handle for the file,
+ *      using the file's encapsulation type, snapshot length,
+ *      and time stamp resolution, and add it to the interface
+ *      data for a file.
+ * @note This requires that the encapsulation type and time stamp
+ *      resolution not be per-packet; it will terminate the process
+ *      if either of them are.
+ *
+ * @param wth The wiretap handle for the file.
+ */
+WS_DLL_PUBLIC
+void wtap_add_generated_idb(wtap *wth);
+
 /**
  * @brief Gets existing interface descriptions.
  * @details Returns a new struct containing a pointer to the existing
@@ -1891,6 +1978,18 @@ void wtap_write_shb_comment(wtap *wth, gchar *comment);
 WS_DLL_PUBLIC
 wtapng_iface_descriptions_t *wtap_file_get_idb_info(wtap *wth);
 
+/**
+ * @brief Gets next interface description.
+ *
+ * @details This returns the first unfetched wtap_block_t from the set
+ * of interface descriptions.  Returns NULL if there are no more
+ * unfetched interface descriptions; a subsequent call after
+ * wtap_read() returns, either with a new record or an EOF, may return
+ * another interface description.
+ */
+WS_DLL_PUBLIC
+wtap_block_t wtap_get_next_interface_description(wtap *wth);
+
 /**
  * @brief Free's a interface description block and all of its members.
  *
@@ -2010,6 +2109,23 @@ gboolean wtap_dump_supports_comment_types(int filetype, guint32 comment_types);
 WS_DLL_PUBLIC
 void wtap_dump_params_init(wtap_dump_params *params, wtap *wth);
 
+/**
+ * Initialize the per-file information based on an existing file, but
+ * don't copy over the interface information. Its contents must be freed
+ * according to the requirements of wtap_dump_params.
+ * If wth does not remain valid for the duration of the session, dsbs_growing
+ * MUST be cleared after this function.
+ *
+ * XXX - this should eventually become wtap_dump_params_init(), with all
+ * programs writing capture files copying IDBs over by hand, so that they
+ * handle IDBs in the middle of the file.
+ *
+ * @param params The parameters for wtap_dump_* to initialize.
+ * @param wth The wiretap session.
+ */
+WS_DLL_PUBLIC
+void wtap_dump_params_init_no_idbs(wtap_dump_params *params, wtap *wth);
+
 /**
  * Remove any decryption secret information from the per-file information;
  * used if we're stripping decryption secrets as we write the file.
@@ -2037,12 +2153,14 @@ void wtap_dump_params_cleanup(wtap_dump_params *params);
  * @param compression_type Type of compression to use when writing, if any
  * @param params The per-file information for this file.
  * @param[out] err Will be set to an error code on failure.
+ * @param[out] err_info for some errors, a string giving more details of
+ * the error
  * @return The newly created dumper object, or NULL on failure.
  */
 WS_DLL_PUBLIC
 wtap_dumper* wtap_dump_open(const char *filename, int file_type_subtype,
     wtap_compression_type compression_type, const wtap_dump_params *params,
-    int *err);
+    int *err, gchar **err_info);
 
 /**
  * @brief Creates a dumper for a temporary file.
@@ -2054,12 +2172,14 @@ wtap_dumper* wtap_dump_open(const char *filename, int file_type_subtype,
  * @param compression_type Type of compression to use when writing, if any
  * @param params The per-file information for this file.
  * @param[out] err Will be set to an error code on failure.
+ * @param[out] err_info for some errors, a string giving more details of
+ * the error
  * @return The newly created dumper object, or NULL on failure.
  */
 WS_DLL_PUBLIC
 wtap_dumper* wtap_dump_open_tempfile(char **filenamep, const char *pfx,
     int file_type_subtype, wtap_compression_type compression_type,
-    const wtap_dump_params *params, int *err);
+    const wtap_dump_params *params, int *err, gchar **err_info);
 
 /**
  * @brief Creates a dumper for an existing file descriptor.
@@ -2069,12 +2189,14 @@ wtap_dumper* wtap_dump_open_tempfile(char **filenamep, const char *pfx,
  * @param compression_type Type of compression to use when writing, if any
  * @param params The per-file information for this file.
  * @param[out] err Will be set to an error code on failure.
+ * @param[out] err_info for some errors, a string giving more details of
+ * the error
  * @return The newly created dumper object, or NULL on failure.
  */
 WS_DLL_PUBLIC
 wtap_dumper* wtap_dump_fdopen(int fd, int file_type_subtype,
     wtap_compression_type compression_type, const wtap_dump_params *params,
-    int *err);
+    int *err, gchar **err_info);
 
 /**
  * @brief Creates a dumper for the standard output.
@@ -2083,18 +2205,25 @@ wtap_dumper* wtap_dump_fdopen(int fd, int file_type_subtype,
  * @param compression_type Type of compression to use when writing, if any
  * @param params The per-file information for this file.
  * @param[out] err Will be set to an error code on failure.
+ * @param[out] err_info for some errors, a string giving more details of
+ * the error
  * @return The newly created dumper object, or NULL on failure.
  */
 WS_DLL_PUBLIC
 wtap_dumper* wtap_dump_open_stdout(int file_type_subtype,
     wtap_compression_type compression_type, const wtap_dump_params *params,
-    int *err);
+    int *err, gchar **err_info);
 
 WS_DLL_PUBLIC
+gboolean wtap_dump_add_idb(wtap_dumper *wdh, wtap_block_t idb, int *err,
+     gchar **err_info);
+WS_DLL_PUBLIC
 gboolean wtap_dump(wtap_dumper *, const wtap_rec *, const guint8 *,
      int *err, gchar **err_info);
 WS_DLL_PUBLIC
-void wtap_dump_flush(wtap_dumper *);
+gboolean wtap_dump_flush(wtap_dumper *, int *);
+WS_DLL_PUBLIC
+int wtap_dump_file_type_subtype(wtap_dumper *wdh);
 WS_DLL_PUBLIC
 gint64 wtap_get_bytes_dumped(wtap_dumper *);
 WS_DLL_PUBLIC
@@ -2114,7 +2243,7 @@ void wtap_dump_discard_decryption_secrets(wtap_dumper *wdh);
  * shb_hdr, idb_inf and nrb_hdr are not freed by this routine.
  */
 WS_DLL_PUBLIC
-gboolean wtap_dump_close(wtap_dumper *wdh, int *err);
+gboolean wtap_dump_close(wtap_dumper *wdh, int *err, gchar **err_info);
 
 /**
  * Return TRUE if we can write a file out with the given GArray of file
@@ -2132,6 +2261,13 @@ WS_DLL_PUBLIC
 GArray *wtap_get_savable_file_types_subtypes(int file_type,
     const GArray *file_encaps, guint32 required_comment_types);
 
+/**
+ * Return TRUE if files of this file type/subtype use interface IDs
+ * to associate records with an interface.
+ */
+WS_DLL_PUBLIC
+gboolean wtap_uses_interface_ids(int file_type);
+
 /*** various string converter functions ***/
 WS_DLL_PUBLIC
 const char *wtap_file_type_subtype_string(int file_type_subtype);
@@ -2209,7 +2345,7 @@ void wtap_deregister_file_type_subtype(const int file_type_subtype);
 WS_DLL_PUBLIC
 int wtap_register_encap_type(const char *description, const char *name);
 
-/*** Cleanup the interal library structures */
+/*** Cleanup the internal library structures */
 WS_DLL_PUBLIC
 void wtap_cleanup(void);
 
@@ -2305,7 +2441,7 @@ void wtap_cleanup(void);
 #endif /* __WTAP_H__ */
 
 /*
- * Editor modelines  -  http://www.wireshark.org/tools/modelines.html
+ * Editor modelines  -  https://www.wireshark.org/tools/modelines.html
  *
  * Local variables:
  * c-basic-offset: 4
